@@ -38,7 +38,7 @@
 #include <kiconloader.h>
 #include <kdebug.h>
 #include <kurl.h>
-
+#include <kmimetype.h>
 #include <qlayout.h>
 #include <qdir.h>
 #include <qfileinfo.h>
@@ -131,7 +131,10 @@ KDevProjectManagerWidget::KDevProjectManagerWidget(KDevProjectManagerPart *part)
         
     m_addFolder = new KAction(i18n("Add Folder"), SmallIcon("folder"), 0, this, SLOT(createFolder()),
         part->actionCollection(), "project_add_folder");
-
+     
+    m_actionConfigure = new KAction(i18n("Configure"), SmallIcon("configure"), 0, this, SLOT(configureFolder()),
+        part->actionCollection(), "project_configure_folder");
+	
     QSplitter *splitter = new QSplitter(Qt::Vertical, this);
     m_overview = new ProjectOverview(this, splitter);
     m_details = new ProjectDetails(this, splitter);
@@ -225,6 +228,7 @@ void KDevProjectManagerWidget::createFolder()
         QFileInfo fileInfo(activeFolder()->name() + "/" + name);
         
         if (QDir::current().mkdir(fileInfo.absFilePath())) {
+            editor->createProjectFile( fileInfo.absFilePath() );
             ProjectItemDom item = editor->import(projectModel(), fileInfo.absFilePath());
             
             if (item && item->toFolder()) {                
@@ -243,8 +247,18 @@ void KDevProjectManagerWidget::createFolder()
 
 void KDevProjectManagerWidget::createTarget()
 {
+	KDevProjectEditor *editor = part()->defaultImporter()->editor();
+	QString name = KInputDialog::getText(i18n("Add Target"), i18n("Add target:"));
+	if (!name.isEmpty()) {
+		// Create the target in the AST and then add it to the DOM
+	}
 }
 
+void KDevProjectManagerWidget::configureFolder()
+{
+	KDevProjectEditor *editor = part()->defaultImporter()->editor();
+	editor->configureFolder(activeFolder());
+}
 // ---------------------------------------------------------------------------------
 class ProjectRoot: public ProjectViewItem
 {
@@ -282,6 +296,7 @@ ProjectView::ProjectView(KDevProjectManagerWidget *m, QWidget *parentWidget)
     m_listView->addColumn(QString::null);
     m_listView->setRootIsDecorated(QListView::LastColumn);
     m_listView->setResizeMode(QListView::LastColumn);
+    m_listView->setSorting(-1);
     
     connect(m_listView, SIGNAL(returnPressed(QListViewItem*)), this, SLOT(executed(QListViewItem*)));
     connect(m_listView, SIGNAL(executed(QListViewItem*)), this, SLOT(executed(QListViewItem*)));    
@@ -430,15 +445,21 @@ void ProjectViewItem::setup()
 {
     QListViewItem::setup();
     
-    if (dom()) {    
-        if (ProjectWorkspaceDom workspace = dom()->toWorkspace())
+    if (dom()) {
+        if (dom()->hasAttribute("Icon"))
+             setPixmap(0, SmallIcon(dom()->attribute("Icon").toString()));
+        else if (ProjectWorkspaceDom workspace = dom()->toWorkspace())
             setPixmap(0, SmallIcon("window"));
-        else if (ProjectFolderDom folder = dom()->toFolder())
-            setPixmap(0, SmallIcon("folder"));
         else if (ProjectTargetDom target = dom()->toTarget())
             setPixmap(0, SmallIcon("target_kdevelop"));
+        else if (ProjectFolderDom folder = dom()->toFolder())
+                setPixmap(0, SmallIcon("folder"));
         else if (ProjectFileDom file = dom()->toFile())
-            setPixmap(0, SmallIcon("document"));
+        {
+	        setPixmap(0, KMimeType::pixmapForURL( file->name(), 0, KIcon::Small ) );
+        }
+        else
+                setPixmap(0, SmallIcon("document")); // Use kio's mime type to get file type.
     }
 }
 
@@ -462,6 +483,7 @@ ProjectOverview::ProjectOverview(KDevProjectManagerWidget *manager, QWidget *par
         part()->actionCollection()->action("project_buildall")->plug(tb);
         tb->insertSeparator();
         part()->actionCollection()->action("project_reload")->plug(tb);
+        part()->actionCollection()->action("project_configure_folder")->plug(tb);
         
 #if 0 // ###
         tb->insertButton(SmallIcon("folder_new"), -1, true);
@@ -712,7 +734,8 @@ void ProjectOverview::contextMenu(KListView *listView, QListViewItem *item, cons
             urls.append(folder->name());
             FileContext fileContext(urls);
             part()->core()->fillContextMenu(&menu, &fileContext);
-
+            part()->defaultImporter()->fillContextMenu( &menu, &fileContext );
+	        
             menu.insertItem(i18n("Edit"), 1000);
             
             if (part()->defaultBuilder()) {
@@ -742,35 +765,41 @@ void ProjectDetails::contextMenu(KListView *listView, QListViewItem *item, const
     
     if (!item)
         return;
-        
-    ProjectViewItem *projectItem = static_cast<ProjectViewItem*>(item);
-        
-    if (ProjectFileDom file = projectItem->dom()->toFile()) {
-        KPopupMenu menu(this);
-        menu.insertTitle(i18n("File: %1").arg(file->shortDescription()));
-                    
-        ProjectModelItemContext context(file.data());
-        part()->core()->fillContextMenu(&menu, &context);
-        
-        KURL::List urls;
-        urls.append(file->name());
-        FileContext fileContext(urls);
-        part()->core()->fillContextMenu(&menu, &fileContext);
-        
-        if (part()->defaultBuilder()) {
-            menu.insertSeparator();
-            menu.insertItem(i18n("Build"), 1010);
-        }
-        
-        switch (menu.exec(pt)) {
-            case 1010: {
-                if (KDevProjectBuilder *builder = part()->defaultBuilder())
-                    builder->build(m_currentItem); // ### TODO: compile the target not subproject
-            } break;
-            
-            default: break;
-        } // end switch
-    }
+	KPopupMenu menu(this);
+	ProjectViewItem *projectItem = static_cast<ProjectViewItem*>(item);
+	if (ProjectTargetDom target = projectItem->dom()->toTarget()) {
+		
+		menu.insertTitle(i18n("Target: %1").arg(target->shortDescription()));
+		ProjectModelItemContext context(target.data());
+		part()->defaultImporter()->fillContextMenu(&menu, &context);
+	}
+	else if (ProjectFileDom file = projectItem->dom()->toFile()) {
+		menu.insertTitle(i18n("File: %1").arg(file->shortDescription()));
+		ProjectModelItemContext context(file.data());
+		part()->core()->fillContextMenu(&menu, &context);
+		KURL::List urls;
+		urls.append(file->name());
+		FileContext fileContext(urls);
+		part()->core()->fillContextMenu(&menu, &fileContext);
+		part()->defaultImporter()->fillContextMenu( &menu, &fileContext );
+    	}
+	
+	if (part()->defaultBuilder()) {
+		menu.insertSeparator();
+		menu.insertItem(i18n("Build"), 1010);
+	}
+
+	if( menu.count() != 0)
+	{
+		switch (menu.exec(pt)) {
+		case 1010: {
+			if (KDevProjectBuilder *builder = part()->defaultBuilder())
+				builder->build(m_currentItem); // ### TODO: compile the target not subproject
+		} break;
+			
+		default: break;
+		} // end switch
+	}
 }
 
 
