@@ -21,13 +21,13 @@
 #include <qmap.h>
 #include <qdatetime.h>
 
-#include "kdevproject.h"
+#include "kdevbuildtool.h"
 
 class KDialogBase;
 class TrollProjectWidget;
 
 
-class TrollProjectPart : public KDevProject
+class TrollProjectPart : public KDevBuildTool
 {
     Q_OBJECT
 
@@ -65,6 +65,8 @@ private slots:
 //    void slotClean();
 //    void slotExecute();
     void slotCommandFinished( const QString& command );
+    void slotBuildAndExecuteProject();
+    void slotBuildAndExecuteTarget();
 
 private:
 //    void startMakeCommand(const QString &dir, const QString &target);
