@@ -1,5 +1,4 @@
-/* $Id: qeditor_view.cpp,v 1.51 2004/03/04 05:20:42 cunz Exp $
- *
+/*
  *  Copyright (C) 2001 Roberto Raggi (roberto@kdevelop.org)
  *
  *  This program is free software; you can redistribute it and/or
@@ -53,7 +52,7 @@
 #include <kcombobox.h>
 #include <kdeversion.h>
 
-#include "kdevkstdaction.h"
+//#include "kdevkstdaction.h"
 
 QEditorView::QEditorView( QEditorPart* document, QWidget* parent, const char* name )
     : KTextEditor::View( document, parent, name ),
