@@ -21,6 +21,7 @@
 #include <kpopupmenu.h>
 #include <klineedit.h>
 #include <kdeversion.h>
+#include <kiconloader.h>
 
 #include <qheader.h>
 #include <qlabel.h>
@@ -30,12 +31,37 @@
 #include <qpushbutton.h>
 #include <qregexp.h>
 #include <qcursor.h>
+#include <qwhatsthis.h>
 #include <klocale.h>
 
 #include <qpoint.h>
 #include <qclipboard.h>
 #include <kapplication.h>
 
+
+/** The variables widget is passive, and is invoked by the rest of the
+    code via two main slots:
+    - slotDbgStatus
+    - slotCurrentFrame
+
+    The first is received the program status changes and the second is
+    recieved after current frame in the debugger can possibly changes.
+
+    The widget has a list item for each frame/thread combination, with
+    variables as children. However, at each moment only one item is shown.
+    When handling the slotCurrentFrame, we check if variables for the
+    current frame are available. If yes, we simply show the corresponding item.
+    Otherwise, we fetch the new data from debugger.
+
+    Fetching the data is done by emitting the produceVariablesInfo signal.
+    In response, we get slotParametersReady and slotLocalsReady signal,
+    in that order.
+
+    The data is parsed and changed variables are highlighted. After that,
+    we 'trim' variable items that were not reported by gdb -- that is, gone
+    out of scope.
+*/
+
 // **************************************************************************
 // **************************************************************************
 // **************************************************************************
@@ -44,72 +70,93 @@ namespace GDBDebugger
 {
 
 VariableWidget::VariableWidget(QWidget *parent, const char *name)
-    : QWidget(parent, name)
+    : QWidget(parent, name), firstShow_(true)
 {
+    setIcon(SmallIcon("math_brace"));
+    setCaption(i18n("Variable Tree"));
+
     varTree_ = new VariableTree(this);
-    QLabel *label = new QLabel(i18n("E&xpression to watch:"), this);
+    
+    watchVarEditor_ = new KHistoryCombo( this, 
+                                         "var-to-watch editor");
 
-    QHBox *watchEntry = new QHBox( this );
+    QHBoxLayout* buttons = new QHBoxLayout();
 
-    watchVarEditor_ = new KHistoryCombo( watchEntry, "var-to-watch editor");
-    label->setBuddy(watchVarEditor_);
+    buttons->addStretch();
 
-//    watchVarEntry_ = new KLineEdit(this);
+    QPushButton *evalButton = new QPushButton(i18n("&Evaluate"), this );
+    buttons->addWidget(evalButton);   
 
-    QPushButton *addButton = new QPushButton(i18n("&Add"), watchEntry );
-    addButton->adjustSize();
-    addButton->setFixedWidth(addButton->width());
-
-    QBoxLayout * vbox = new QVBoxLayout();
-
-//    QBoxLayout *watchEntry = new QHBoxLayout();
-//    watchEntry->addWidget(label);
-//    watchEntry->addWidget(watchVarEntry_);
-//    watchEntry->addWidget(watchVarEditor_);
-//    watchEntry->setStretchFactor(watchVarEditor_, 1);
-//    watchEntry->addWidget(addButton);
-
-    vbox->addWidget( label );
-    vbox->addWidget( watchEntry );
+    QPushButton *addButton = new QPushButton(i18n("&Watch"), this );
+    buttons->addWidget(addButton);
 
     QVBoxLayout *topLayout = new QVBoxLayout(this, 2);
     topLayout->addWidget(varTree_, 10);
-    topLayout->addLayout( vbox );
+    topLayout->addWidget(watchVarEditor_);
+    topLayout->addItem(buttons);
+    
 
     connect( addButton, SIGNAL(clicked()), SLOT(slotAddWatchVariable()) );
-    connect( watchVarEditor_, SIGNAL(returnPressed()), SLOT(slotAddWatchVariable()) );
-//    connect( watchVarEntry_, SIGNAL(returnPressed()), SLOT(slotAddWatchVariable()) );
+    connect( evalButton, SIGNAL(clicked()), SLOT(slotEvaluateExpression()) );
 
+    connect( watchVarEditor_, SIGNAL(returnPressed()), 
+             SLOT(slotEvaluateExpression()) );
+
+    // Setup help items.
+
+    QWhatsThis::add(this, i18n(
+        "<b>Variable tree</b><p>"
+        "The variable tree allows you to see the values of local "
+        "variables and arbitrary expressions."              
+        "<p>Local variables are displayed automatically and are updated "
+        "as you step through your program. "
+        "For each expression you enter, you can either evaluate it once, "
+        "or \"watch\" it (make it auto-updated). Expressions that are not "
+        "auto-updated can be updated manually from the context menu. "
+        "Expressions can be renamed to more descriptive names by clicking "
+        "on the name column."
+        "<p>To change the value of a variable or an expression, "
+        "click on the value."));
+
+    QWhatsThis::add(watchVarEditor_, 
+                    i18n("<b>Expression entry</b>"
+                         "<p>Type in expression to evaluate."));
+
+    QWhatsThis::add(evalButton, 
+                    i18n("Evaluate the expression."));
+
+    QWhatsThis::add(addButton, 
+                    i18n("Evaluate the expression and "
+                         "auto-update the value when stepping."));
 }
 
 // **************************************************************************
 
 void VariableWidget::clear()
 {
-//  varTree_->clear();
-  QListViewItemIterator it(varTree_);
-  while (it.current())
-  {
-    if (!dynamic_cast<WatchRoot*>(varTree_->findRoot(it.current())))
-    {
-      QListViewItem *item = it.current();
-      delete item;
-    } else
-    {
-      ++it;
-    }
-  }
+  // Use 'trim' so that logic about which top-level items are 
+  // always retained is contained only in that function.
+  varTree_->setActiveFlag();
+  varTree_->trim();
 }
 
 // **************************************************************************
 
-void VariableWidget::setEnabled(bool bEnabled)
+// When the variables view is shown the first time, 
+// set the width of 'variable name' column to half the total
+// width.
+// Ideally, KMDI should emit 'initial size set' signal, but
+// it does not, so we rely on the fact that size is already
+// set when the widget is first shown.
+void VariableWidget::showEvent(QShowEvent *)
 {
-    QWidget::setEnabled(bEnabled);
-    if (bEnabled && parentWidget()) {
-        varTree_->setColumnWidth(0, parentWidget()->width()/2);
-  }
+    if (firstShow_)
+    {
+        firstShow_ = false;
+        varTree_->setColumnWidth(0, width()/2);
+    }
 }
+
 // **************************************************************************
 
 void VariableWidget::slotAddWatchVariable()
@@ -134,6 +181,25 @@ void VariableWidget::slotAddWatchVariable(const QString &ident)
     }
 }
 
+void VariableWidget::slotEvaluateExpression()
+{
+    QString exp(watchVarEditor_->currentText());
+    if (!exp.isEmpty())
+    {
+        slotEvaluateExpression(exp);
+    }
+}
+
+void VariableWidget::slotEvaluateExpression(const QString &ident)
+{
+    if (!ident.isEmpty())
+    {
+        watchVarEditor_->addToHistory(ident);
+        varTree_->slotEvaluateExpression(ident);
+        watchVarEditor_->clearEdit();
+    }    
+}
+
 // **************************************************************************
 
 void VariableWidget::focusInEvent(QFocusEvent */*e*/)
@@ -142,6 +208,8 @@ void VariableWidget::focusInEvent(QFocusEvent */*e*/)
 }
 
 
+
+
 // **************************************************************************
 // **************************************************************************
 // **************************************************************************
@@ -150,7 +218,9 @@ VariableTree::VariableTree(VariableWidget *parent, const char *name)
     : KListView(parent, name),
       QToolTip( viewport() ),
       activeFlag_(0),
-      currentThread_(-1)
+      currentThread_(-1),
+      justPaused_(false),
+      recentExpressions_(0)
 {
     setRootIsDecorated(true);
     setAllColumnsShowFocus(true);
@@ -165,14 +235,8 @@ VariableTree::VariableTree(VariableWidget *parent, const char *name)
     connect( this, SIGNAL(contextMenu(KListView*, QListViewItem*, const QPoint&)),
              SLOT(slotContextMenu(KListView*, QListViewItem*)) );
     connect( this, SIGNAL(toggleRadix(QListViewItem*)), SLOT(slotToggleRadix(QListViewItem*)) );
-
-/*
-    work in progress - disabled for now
-
-    // jw
-    connect( this, SIGNAL(doubleClicked(QListViewItem *item, const QPoint &pos, int c)),
-             SLOT(slotDoubleClicked(QListViewItem *item, const QPoint &pos, int c)) );
-*/
+    connect( this, SIGNAL(itemRenamed( QListViewItem*, int, const QString&)),
+             this, SLOT(slotItemRenamed( QListViewItem*, int, const QString&)));
 }
 
 // **************************************************************************
@@ -192,19 +256,60 @@ void VariableTree::slotContextMenu(KListView *, QListViewItem *item)
 
     if (item->parent())
     {
-        KPopupMenu popup(item->text(VarNameCol), this);
-        int idRemoveWatch = -2;
-        if (dynamic_cast<WatchRoot*>(findRoot(item)))
-            idRemoveWatch = popup.insertItem( i18n("Remove Watch Variable") );
+        KPopupMenu popup(this);
+        popup.insertTitle(item->text(VarNameCol));
+        int idRemember = -2;
+        int idRemove = -2;
+        int idReevaluate = -2;        
+        int idWatch = -2;
+
+        QListViewItem* root = findRoot(item);
+
+        if (root != recentExpressions_)
+        {
+            idRemember = popup.insertItem(
+                SmallIcon("pencil"), i18n("Remember Value"));
+        }
+
+        if (dynamic_cast<WatchRoot*>(root)) {
+            idRemove = popup.insertItem( 
+                SmallIcon("editdelete"), i18n("Remove Watch Variable") );
+        } else if (root != recentExpressions_) {
+            idWatch = popup.insertItem(
+                i18n("Watch Variable"));
+        }
+        if (root == recentExpressions_) {
+            idReevaluate = popup.insertItem( 
+                SmallIcon("reload"), i18n("Reevaluate Expression") );
+            idRemove = popup.insertItem( 
+                SmallIcon("editdelete"), i18n("Remove Expression") );
+        }
 
         int idToggleWatch = popup.insertItem( i18n("Toggle Watchpoint") );
         int idToggleRadix = popup.insertItem( i18n("Toggle Hex/Decimal") );
-        int	idCopyToClipboard = popup.insertItem( i18n("Copy to Clipboard") );
+        int	idCopyToClipboard = popup.insertItem( 
+            SmallIcon("editcopy"), i18n("Copy to Clipboard") );
         int res = popup.exec(QCursor::pos());
 
-        if (res == idRemoveWatch)
+        if (res == idRemember)
+        {
+            if (VarItem *item = dynamic_cast<VarItem*>(currentItem()))
+            {
+                ((VariableWidget*)parent())->
+                    slotEvaluateExpression(item->gdbExpression());
+            }
+        } 
+        else if (res == idWatch)
+        {
+            if (VarItem *item = dynamic_cast<VarItem*>(currentItem()))
+            {
+                ((VariableWidget*)parent())->
+                    slotAddWatchVariable(item->gdbExpression());
+            }
+        } 
+        else if (res == idRemove)
             delete item;
-        if (res == idToggleRadix)
+        else if (res == idToggleRadix)
             emit toggleRadix(item);
         else if (res == idCopyToClipboard)
         {
@@ -222,7 +327,40 @@ void VariableTree::slotContextMenu(KListView *, QListViewItem *item)
         else if (res == idToggleWatch)
         {
             if (VarItem *item = dynamic_cast<VarItem*>(currentItem()))
-                emit toggleWatchpoint(item->fullName());
+                emit toggleWatchpoint(item->gdbExpression());
+        }
+        else if (res == idReevaluate)
+        {
+            if (VarItem* item = dynamic_cast<VarItem*>(currentItem()))
+            {
+                emit expandItem(item);
+            }
+        }
+    }
+    else if (item == recentExpressions_)
+    {
+        KPopupMenu popup(this);
+        popup.insertTitle(i18n("Recent Expressions"));
+        int idRemove = popup.insertItem(
+            SmallIcon("editdelete"), i18n("Remove All"));
+        int idReevaluate = popup.insertItem(
+            SmallIcon("reload"), i18n("Reevaluate All"));
+        int res = popup.exec(QCursor::pos());
+        
+        if (res == idRemove)
+        {
+            delete recentExpressions_;
+            recentExpressions_ = 0;
+        }
+        else if (res == idReevaluate)
+        {
+            for(QListViewItemIterator it(item); *it; ++it)
+            {
+                VarItem* var = dynamic_cast<VarItem*>(*it);                    
+                Q_ASSERT(var && 
+                         "only VarItem allowed under 'Recent expressions'");
+                emit expandItem(var);
+            }
         }
     }
 }
@@ -236,49 +374,20 @@ void VariableTree::slotAddWatchVariable(const QString &watchVar)
     emit expandItem(varItem);
 }
 
-// **************************************************************************
-
-// jw
-void VariableTree::slotDoubleClicked(QListViewItem *item, const QPoint &pos, int c)
+void VariableTree::slotEvaluateExpression(const QString &expression)
 {
-    kdDebug(9012) << " ### VariableTree::slotDoubleClicked 1" << endl;
-
-    if (item)
+    if (recentExpressions_ == 0)
     {
-        kdDebug(9012) << " ### VariableTree::slotDoubleClicked 2" << endl;
-        TrimmableItem *titem = dynamic_cast<TrimmableItem*>(item);
-        if (titem)
-        {
-            kdDebug(9012) << " ### VariableTree::slotDoubleClicked 2" << endl;
-            titem->handleDoubleClicked(pos, c);
-        }
-    }
-}
-
-// **************************************************************************
-
-void VariableTree::setLocalViewState(bool localsOn, int frameNo, int threadNo)
-{
-    // When they want to _close_ a frame then we need to check the state of
-    // all other frames to determine whether we still need the locals.
-    if (!localsOn) {
-        QListViewItem *sibling = firstChild();
-        while (sibling) {
-            VarFrameRoot *frame = dynamic_cast<VarFrameRoot*> (sibling);
-            if (frame && frame->isOpen()) {
-                localsOn = true;
-                break;
-            }
-
-            sibling = sibling->nextSibling();
-        }
+        recentExpressions_ = new TrimmableItem(this);
+        recentExpressions_->setText(0, "Recent");
+        recentExpressions_->setOpen(true);
     }
 
-    emit setLocalViewState(localsOn);
-    emit selectFrame(frameNo, threadNo);
+    VarItem *varItem = new VarItem(recentExpressions_, expression, typeUnknown);
+    varItem->setRenameEnabled(0, 1);
+    emit expandItem(varItem);
 }
 
-
 // **************************************************************************
 
 QListViewItem *VariableTree::findRoot(QListViewItem *item) const
@@ -333,8 +442,9 @@ void VariableTree::trim()
     while (child) {
         QListViewItem *nextChild = child->nextSibling();
 
-        // don't trim the watch root
-        if (!(dynamic_cast<WatchRoot*> (child))) {
+        // don't trim the watch root, or 'recent expressions' root.
+        if (!(dynamic_cast<WatchRoot*> (child)) 
+            && child != recentExpressions_) {
             if (TrimmableItem *item = dynamic_cast<TrimmableItem*> (child)) {
                 if (item->isActive())
                     item->trim();
@@ -434,6 +544,138 @@ void VariableTree::slotToggleRadix(QListViewItem * item)
   pOldItem=NULL;
 }
 
+void VariableTree::slotDbgStatus(const QString&, int statusFlag)
+{
+    if (statusFlag & s_appNotStarted)
+    {
+        // The application no longer exists. Remove all locals.
+        setActiveFlag();
+
+        // Now wipe the tree out
+        viewport()->setUpdatesEnabled(false);
+        trim();
+        setUpdatesEnabled(true);
+        repaint();
+    }
+    else
+    {
+        // Application still exists.
+        if (!(statusFlag & s_appBusy))
+        {
+            // But is not busy. This means application has just stopped for
+            // some reason. Need to refresh locals when
+            // slotChangedFrame is called. Cannot do it here, because
+            // we don't know which thread we're in.
+            justPaused_ = true;
+        }
+    }
+}
+
+VarFrameRoot* VariableTree::demand_frame_root(int frameNo, int threadNo)
+{
+    VarFrameRoot *frame = findFrame(frameNo, threadNo); 
+    if (!frame)
+    {
+        frame = new VarFrameRoot(this, frameNo, threadNo);
+        frame->setFrameName("Locals");
+        // Make sure "Locals" item is always the top item, before
+        // "watch" and "recent experessions" items.
+       this->takeItem(frame);
+       this->insertItem(frame);
+    }
+    return frame;
+}
+
+void VariableTree::slotParametersReady(const char* data)
+{
+    // The locals are always attached to the currentFrame
+    VarFrameRoot *frame = demand_frame_root(currentFrame_, currentThread_);
+    frame->setParams(data);
+}
+
+void VariableTree::slotLocalsReady(const char* data)
+{
+    setUpdatesEnabled(false);
+
+    VarFrameRoot *frame = demand_frame_root(currentFrame_, currentThread_);
+    frame->setLocals(data);
+    frame->setOpen(true);
+    
+    // If we're regetting locals for the frame 0, it surely means
+    // the application was just paused. Otherwise, 
+    // (say after selecting frame 1 and then frame 0) we'd have locals
+    // for frame 0 already. If app was just paused, then other frames
+    // are out-of-date, and we trim them. If user later selects frame 1,
+    // we get locals for that frame.
+    // TODO: should we reset data for other threads?
+    if (currentFrame_ == 0 || currentThread_ == -1)
+        trim();
+    else 
+       frame->trim();
+
+    setUpdatesEnabled(true);
+    triggerUpdate();
+}
+
+void VariableTree::slotCurrentFrame(int frameNo, int threadNo)
+{
+    // It's quite likely that frameNo == currentFrame_ and
+    // threadNo == currentThread_. For example, this happens
+    // when the 'step' command is executed.
+    if (frameNo != currentFrame_ || threadNo != currentThread_)
+    {
+        // Hide the current frame vars root.
+        demand_frame_root(currentFrame_, currentThread_)->setVisible(false);
+        
+        currentFrame_ = frameNo;
+        currentThread_ = threadNo;
+    }
+
+    // Show the current frame.
+    VarFrameRoot* frame = demand_frame_root(currentFrame_, currentThread_);
+    frame->setVisible(true);
+        
+    // If no locals for frame were obtained, reget the local.
+    // Also reget the locals if the program was just paused. In that
+    // case we're always on frame 0, and the setLocals function
+    // we eventually remove frames 1, N, if they are present. They
+    // will be repopulated if needed.
+    if (frame->needLocals() || justPaused_) 
+    {
+        setActiveFlag();
+        // This will eventually call back to slotParametersReady and 
+        // slotLocalsReady
+        emit produceVariablesInfo();
+
+        if (justPaused_)
+        {
+            findWatch()->requestWatchVars();
+        }
+        justPaused_ = false;
+    }
+}
+
+void 
+VariableTree::slotItemRenamed(QListViewItem* item, int col, const QString& text)
+{
+    if (col == ValueCol)
+    {
+        VarItem* v = dynamic_cast<VarItem*>(item);
+        Q_ASSERT(v);
+        if (v)
+        {
+            // Set the value
+            emit setValue(v->gdbExpression(), text);
+            // And immediately reload it from gdb, 
+            // so that it's display format is the one gdb uses,
+            // not the one user has typed. Otherwise, on the next
+            // step, the visible value might change and be highlighted
+            // as changed, which is bogus.
+            emit expandItem(v);
+        }
+    }
+}
+
 // **************************************************************************
 // **************************************************************************
 // **************************************************************************
@@ -611,16 +853,14 @@ QString TrimmableItem::key (int, bool) const
 
 VarItem::VarItem(TrimmableItem *parent, const QString &varName, DataType dataType)
     : TrimmableItem (parent),
+      name_(varName),
       cache_(QCString()),
       dataType_(dataType),
       highlight_(false)
 {
     setText(VarNameCol, varName);
-
-/*
-    setRenameEnabled(VarTypeCol, true);
-    setRenameEnabled(VarNameCol, true);
-*/
+    // Allow to change variable name by editing.
+    setRenameEnabled(ValueCol, true);
 
     kdDebug(9012) << " ### VarItem::VarItem *CONSTR*" << endl;
     emit ((VariableTree*)listView())->varItemConstructed(this);
@@ -634,40 +874,37 @@ VarItem::~VarItem()
 
 // **************************************************************************
 
-QString VarItem::varPath() const
+QString VarItem::gdbExpression() const
 {
     QString vPath("");
-    const VarItem *item = this;
+    for(const VarItem* item = this; 
+        item; 
+        item = dynamic_cast<const VarItem*>(item->parent()))
+    {
+        // Children of array item have array names in them,
+        // e.g. "p[0]", so when visiting parent we don't need to
+        // add parent name. However, when 'gdbExpression' is called
+        // on array itself, we do need the name.
+        if (item->getDataType() == typeArray && item != this)
+            continue;
 
-    // This stops at the root item (FrameRoot or WatchRoot)
-    while ((item = dynamic_cast<const VarItem*> (item->parent()))) {
-        if (item->getDataType() != typeArray) {
-            if ((item->text(VarNameCol))[0] != '<') {
-                QString itemName = item->text(VarNameCol);
-                if (vPath.isEmpty())
-                    vPath = itemName.replace(QRegExp("^static "), "");
-                else
-                    vPath = itemName.replace(QRegExp("^static "), "") + "." + vPath;
-            }
+        // VP, 2005/07/19: I don't know the reason for this
+        // check. But retaining to avoid breaking anything.
+        if ((item->text(VarNameCol))[0] != '<') {
+            QString itemName = item->name_;
+            if (vPath.isEmpty())
+                vPath = itemName.replace(QRegExp("^static "), "");
+            else
+                vPath = itemName.replace(QRegExp("^static "), "") 
+                    + "." + vPath;
         }
     }
 
-    return vPath;
-}
-
-// **************************************************************************
-
-QString VarItem::fullName() const
-{
-    QString itemName = getName();
-    QString vPath = varPath();
-    if (itemName[0] == '<')
+    if (isOpen() && dataType_ == typePointer)
+        // We're currently showing pointed-to value        
+        return "*" + vPath;
+    else
         return vPath;
-
-    if (vPath.isEmpty())
-        return itemName.replace(QRegExp("^static "), "");
-
-    return varPath() + "." + itemName.replace(QRegExp("^static "), "");
 }
 
 // **************************************************************************
@@ -718,8 +955,6 @@ void VarItem::updateValue(char *buf)
 
     if (dataType_ == typeUnknown) {
         dataType_ = GDBParser::getGDBParser()->determineType(buf);
-        if (dataType_ == typeArray)
-            buf++;
 
         // Try fixing a format string here by overriding the dataType calculated
         // from this data
@@ -728,7 +963,24 @@ void VarItem::updateValue(char *buf)
             dataType_ = typeValue;
     }
 
-    GDBParser::getGDBParser()->parseData(this, buf, true, false);
+    // A hack to nicely display QStrings. The content of QString is unicode
+    // for for ASCII only strings we get ascii character mixed with \000.
+    // Remove those \000 now.
+    
+    // This is not very nice, becuse we're doing this unconditionally
+    // and this method can be called twice: first with data that gdb sends
+    // for a variable, and second after we request the string data. In theory
+    // the data sent by gdb might contain \000 that should not be translated.
+    //
+    // What's even worse, ideally we should convert the string data from
+    // gdb into a QString again, handling all other escapes and composing
+    // one QChar from two characters from gdb. But to do that, we *should*
+    // now if the data if generic gdb value, and result of request for string
+    // data. Fixing is is for later.
+    QCString r(buf);
+    r.replace( QRegExp("\\\\000|\\\\0"), "" );
+
+    GDBParser::getGDBParser()->parseValue(this, r.data());
     setActive();
 }
 
@@ -748,18 +1000,7 @@ void VarItem::updateType(char *buf)
     originalValueType_ = str.latin1();
 
     setText(VarTypeCol, str);
-}
-
-// **************************************************************************
-
-void VarItem::handleDoubleClicked(const QPoint &/*pos*/, int c)
-{
-    kdDebug(9012) << " ### VarItem::handleDoubleClicked 1" << endl;
-    if (c == VarTypeCol || c == ValueCol)
-    {
-        kdDebug(9012) << " ### VarItem::handleDoubleClicked 2" << endl;
-        static_cast<KListView*>(listView())->rename(this, c);
-    }
+    handleSpecialTypes();    
 }
 
 // **************************************************************************
@@ -768,9 +1009,10 @@ void VarItem::setCache(const QCString &value)
 {
     cache_ = value;
     setExpandable(true);
-    checkForRequests();
+    handleSpecialTypes();
     if (isOpen())
         setOpen(true);
+    repaint();
     setActive();
 }
 
@@ -778,11 +1020,17 @@ void VarItem::setCache(const QCString &value)
 
 void VarItem::setOpen(bool open)
 {
+    QListViewItem::setOpen(open);
+
     if (open) {
+        // Opening an item can be potentially expensive, so cache value
+        // received from gdb. If we have value already, just parse it,
+        // don't issue another request.
         if (cache_) {
             QCString value = cache_;
             cache_ = QCString();
-            GDBParser::getGDBParser()->parseData(this, value.data(), false, false);
+            GDBParser::getGDBParser()->parseCompositeValue(this, value.data());
+            handleSpecialTypes();
             trim();
         } else {
             if (dataType_ == typePointer || dataType_ == typeReference) {
@@ -790,9 +1038,14 @@ void VarItem::setOpen(bool open)
                 emit ((VariableTree*)listView())->expandItem(this);
             }
         }
+    } else {
+        // Closing item. For pointer/references, it means we switch from
+        // display the pointer-to value to displaying the pointer itself.
+        if (dataType_ == typePointer || dataType_ == typeReference) {
+            waitingForData();
+            emit ((VariableTree*)listView())->expandItem(this);
+        }
     }
-
-    QListViewItem::setOpen(open);
 }
 
 // **************************************************************************
@@ -804,54 +1057,54 @@ QCString VarItem::getCache()
 
 // **************************************************************************
 
-void VarItem::checkForRequests()
+/* This function is called in two cases -- when the type of variable first
+   becomes known.in two cases. First is when the type of variable
+   first becomes known. Second is when value has changed.
+
+   The method looks at the type, and if necessary, issues an additional gdb
+   requests. 
+*/
+void VarItem::handleSpecialTypes()
 {
-    // This shouldn't be needed to keep it from blowing up, but sometimes is.
-    // On the other hand, if it's empty, there is no reason to go on...
-    if ( cache_.isEmpty() ) return;
+    if (originalValueType_.isEmpty())
+        return;
 
-    /// @todo - hardcoded for now - these should get read from config
+    QString type = originalValueType_;
+    if (dataType_ == typeReference)
+    {
+        // Disable the special processing for non-opened
+        // references. Not sure it's the best thing, but
+        // previous code worked like this.
+        if (!isOpen())
+            return;
 
-    // Signature for a QT1.44 QString
-    if (strncmp(cache_, "<QArrayT<char>> = {<QGArray> = {shd = ", 38) == 0) {
-        waitingForData();
-        emit ((VariableTree*)listView())->expandUserItem(this,
-                                                         fullName().latin1()+QCString(".shd.data"));
+        static QRegExp strip_reference("(.*)[ ]*&");
+        if (strip_reference.exactMatch(type))
+        {
+            type = strip_reference.cap(1);
+        }
+    }
+    if (dataType_ == typePointer)
+    {
+        if (!isOpen())
+            return;
+
+        static QRegExp strip_pointer("(.*)[ ]*\\*");
+        if (strip_pointer.exactMatch(type))
+        {
+            type = strip_pointer.cap(1);
+        }
     }
 
-    // Signature for a QT1.44 QDir
-    if (strncmp(cache_, "dPath = {<QArrayT<char>> = {<QGArray> = {shd", 44) == 0) {
-        waitingForData();
-        emit ((VariableTree*)listView())->expandUserItem(this,
-                                                         fullName().latin1()+QCString(".dPath.shd.data"));
-    }
+    static QRegExp qstring("^(const)?[ ]*QString[ ]*$");
 
-    // Signature for a QT2.x QT3.x QString
-    /// @todo - This handling is not that good - but it works sufficiently well
-    // at the moment to leave it here, and it won't cause bad things to happen.
-    if (strncmp(cache_, "d = 0x", 6) == 0) {     // Eeeek - too small
+    if (qstring.exactMatch(type)) {
         waitingForData();
-        emit ((VariableTree*)listView())->expandUserItem(this,
-                                                         // QCString().sprintf("(($len=($data=%s.d).len)?$data.unicode.rw@($len>100?200:$len*2):\"\")",
-                                                         QCString().sprintf("(($len=($data=%s.d).len)?*((char*)&$data.unicode[0])@($len>100?200:$len*2):\"\")",
-                                                                            fullName().latin1()));
+        emit ((VariableTree*)listView())->expandUserItem(
+            this,
+            QCString().sprintf("(($len=($data=%s.d).len)?*((char*)&$data.unicode[0])@($len>100?200:$len*2):\"\")",
+                               gdbExpression().latin1()));
     }
-
-    // Signature for a QT2.0.x QT2.1 QCString
-    if (strncmp(cache_, "<QArray<char>> = {<QGArray> = {shd = ", 37) == 0) {
-        waitingForData();
-        emit ((VariableTree*)listView())->expandUserItem(this,
-                                                         fullName().latin1()+QCString(".shd.data"));
-    }
-
-    // Signature for a QT2.0.x QT2.1 QDir
-    if (strncmp(cache_, "dPath = {d = 0x", 15) == 0) {
-        waitingForData();
-        ((VariableTree*)listView())->expandUserItem(this,
-                                                    // QCString().sprintf("(($len=($data=%s.dPath.d).len)?$data.unicode.rw@($len>100?200:$len*2):\"\")",
-                                                    QCString().sprintf("(($len=($data=%s.dPath.d).len)?*((char*)&$data.unicode[0])@($len>100?200:$len*2):\"\")",
-                                                                       fullName().latin1()));
-  }
 }
 
 // **************************************************************************
@@ -883,8 +1136,7 @@ void VarItem::paintCell(QPainter *p, const QColorGroup &cg,
 QString VarItem::tipText() const
 {
     const unsigned int maxTooltipSize = 70;
-    /// \FIXME Column #1 is "Value": perhaps some kind of const somewhere is better ...
-    QString tip = text( 1 );
+    QString tip = text( ValueCol );
 
     if (tip.length() < maxTooltipSize )
 	    return tip;
@@ -915,15 +1167,18 @@ VarFrameRoot::~VarFrameRoot()
 
 // **************************************************************************
 
-void VarFrameRoot::setParams(char *params)
+void VarFrameRoot::setParams(const char *params)
 {
     setActive();
-    params_ = params;
+    if (strncmp(params, "No arguments", strlen("No arguments")) == 0)
+        params_ = "";
+    else
+        params_ = params;
 }
 
 // **************************************************************************
 
-void VarFrameRoot::setLocals(char *locals)
+void VarFrameRoot::setLocals(const char *locals)
 {
     setActive();
 
@@ -953,17 +1208,15 @@ void VarFrameRoot::setLocals(char *locals)
 // state. This
 void VarFrameRoot::setOpen(bool open)
 {
-    bool localStateChange = (isOpen() != open);
     QListViewItem::setOpen(open);
 
-    if (localStateChange)
-        ((VariableTree*)listView())->setLocalViewState(open, frameNo_, threadNo_);
-
     if (!open)
         return;
 
-    GDBParser::getGDBParser()->parseData(this, params_.data(), false, true);
-    GDBParser::getGDBParser()->parseData(this, locals_.data(), false, false);
+    if (!params_.isNull())
+        GDBParser::getGDBParser()->parseCompositeValue(this, params_.data());
+    if (!locals_.isNull())
+        GDBParser::getGDBParser()->parseCompositeValue(this, locals_.data());
 
     locals_ = QCString();
     params_ = QCString();
