@@ -133,7 +133,7 @@ void DistpartDialog::slotcreateSrcArchPushButtonPressed() {
     if ( tar.open(IO_WriteOnly) )
     {
     	QStringList files = m_part->project()->distFiles();
-	KProgressDialog *prog = new KProgressDialog( m_part->widget(), "dialog", i18n("Building Package"), "", true );
+	KProgressDialog *prog = new KProgressDialog( 0, "dialog", i18n("Building Package"), "", true );
 	prog->show();
 	for( uint idx = 0; idx < files.count(); ++idx)
 	{
@@ -143,14 +143,14 @@ void DistpartDialog::slotcreateSrcArchPushButtonPressed() {
 		}
 		else
 		{
-			prog->setLabel(i18n("Adding file: ") + files[idx]);
+			prog->setLabel(i18n("Adding file: %1").arg( files[idx]) );
 			prog->progressBar()->setValue( (idx*100)/files.count() );
 		}
 	}
     	tar.close( );
 	prog->hide();
 	delete prog;
-	KMessageBox::information(this, "Archive made at: " + filename,"Archive Done");
+	KMessageBox::information( this, i18n("Archive made at: %1").arg( filename ), i18n("Archive Done") );
 
     }
     else
