@@ -25,22 +25,31 @@
 #include "kdevproject.h"
 #include "kdevpartcontroller.h"
 #include "kdevappfrontend.h"
+#include "kdevplugininfo.h"
+#include "urlutil.h"
+#include "configwidgetproxy.h"
 
 #include "toolsconfig.h"
 #include "toolsconfigwidget.h"
 
+#define TOOLSSETTINGS 1
+#define EXTRATOOLSSETTINGS 2
 
-static const KAboutData data("kdevtools", I18N_NOOP("External Tools"), "1.0");
-K_EXPORT_COMPONENT_FACTORY( libkdevtools, ToolsFactory( &data ) )
+static const KDevPluginInfo data("kdevtools");
+K_EXPORT_COMPONENT_FACTORY( libkdevtools, ToolsFactory( data ) )
 
 ToolsPart::ToolsPart(QObject *parent, const char *name, const QStringList &)
-	: KDevPlugin( "Tools", "configure", parent, name ? name : "ToolsPart")
+	: KDevPlugin( &data, parent, name ? name : "ToolsPart")
 {
   setInstance(ToolsFactory::instance());
 
   setXMLFile("kdevpart_tools.rc");
 
-  connect(core(), SIGNAL(configWidget(KDialogBase*)), this, SLOT(configWidget(KDialogBase*)));
+	m_configProxy = new ConfigWidgetProxy( core() );
+	m_configProxy->createGlobalConfigPage( i18n("Tools Menu"), TOOLSSETTINGS, info()->icon() );
+	m_configProxy->createGlobalConfigPage( i18n("External Tools"), EXTRATOOLSSETTINGS, info()->icon() );
+	connect( m_configProxy, SIGNAL(insertConfigWidget(const KDialogBase*, QWidget*, unsigned int )),
+		this, SLOT(insertConfigWidget(const KDialogBase*, QWidget*, unsigned int )) );
 
   connect(core(), SIGNAL(coreInitialized()), this, SLOT(updateMenu()));
 
@@ -57,22 +66,22 @@ ToolsPart::~ToolsPart()
 {
 }
 
-
-void ToolsPart::configWidget(KDialogBase *dlg)
+void ToolsPart::insertConfigWidget( const KDialogBase * dlg, QWidget * page, unsigned int pagenumber )
 {
-	QVBox *vbox = dlg->addVBoxPage( i18n("Tools Menu"), i18n("Tools Menu"), BarIcon( icon(), KIcon::SizeMedium) );
-  ToolsConfig *w = new ToolsConfig(vbox, "tools config widget");
-  connect(dlg, SIGNAL(okClicked()), w, SLOT(accept()));
-  connect(dlg, SIGNAL(destroyed()), this, SLOT(updateMenu()));
-
-  vbox = dlg->addVBoxPage(i18n("External Tools"), i18n("External Tools"), BarIcon( icon(), KIcon::SizeMedium) );
-  ToolsConfigWidget *w2 = new ToolsConfigWidget(vbox, "tools config widget");
-  connect(dlg, SIGNAL(okClicked()), w2, SLOT(accept()));
-  connect(dlg, SIGNAL(destroyed()), this, SLOT(updateToolsMenu()));
+	if ( pagenumber == TOOLSSETTINGS )
+	{
+		ToolsConfig *w = new ToolsConfig( page, "tools config widget" );
+		connect(dlg, SIGNAL(okClicked()), w, SLOT(accept()));
+		connect(dlg, SIGNAL(destroyed()), this, SLOT(updateMenu()));
+	}
+	else if ( pagenumber == EXTRATOOLSSETTINGS )
+	{
+		ToolsConfigWidget *w2 = new ToolsConfigWidget( page, "tools config widget" );
+		connect(dlg, SIGNAL(okClicked()), w2, SLOT(accept()));
+		connect(dlg, SIGNAL(destroyed()), this, SLOT(updateToolsMenu()));
+	}
 }
 
-
-
 void ToolsPart::updateMenu()
 {
   QPtrList<KAction> actions;
@@ -169,8 +178,12 @@ void ToolsPart::startCommand(QString cmdline, bool captured, QString fileName)
     cmdline.replace(QRegExp("%W"), word);
 
     if (captured)
-        appFrontend()->startAppCommand(QString::QString(), cmdline, false);
-    else {
+    {
+       if (KDevAppFrontend *appFrontend = extension<KDevAppFrontend>("KDevelop/AppFrontend"))
+            appFrontend->startAppCommand(QString::QString(), cmdline, false);
+    }
+    else 
+    {
         KShellProcess proc;
         proc << cmdline;
         proc.start(KProcess::DontCare, KProcess::NoCommunication);
@@ -213,13 +226,13 @@ void ToolsPart::contextMenu(QPopupMenu *popup, const Context *context)
 
     const FileContext *fcontext = static_cast<const FileContext*>(context);
     m_contextPopup = popup;
-    m_contextFileName = fcontext->fileName();
+    m_contextFileName = fcontext->urls().first().path();
     
     KConfig *config = ToolsFactory::instance()->config();
     config->setGroup("External Tools");
     QStringList filecontextList = config->readListEntry("File Context");
 
-    if (fcontext->isDirectory()) {
+    if (URLUtil::isDirectory(m_contextFileName)) {
         QStringList l = config->readListEntry("Dir Context");
         QStringList::ConstIterator it;
         for (it = l.begin(); it != l.end(); ++it)
@@ -260,7 +273,7 @@ void ToolsPart::fileContextActivated(int id)
     QString cmdline = config->readPathEntry("CommandLine");
     bool captured = config->readBoolEntry("Captured");
     kdDebug() << "activating " << menutext
-              << "with cmdline " << cmdline
+              << " with cmdline " << cmdline
               << " on file " << m_contextFileName << endl;
     startCommand(cmdline, captured, m_contextFileName);
 }
