@@ -1,5 +1,5 @@
-/* Copyright (C) 2003
-	 Mickael Marchand <marchand@kde.org>
+/**
+	 Copyright (C) 2003-2005 Mickael Marchand <marchand@kde.org>
 
 	 This program is free software; you can redistribute it and/or
 	 modify it under the terms of the GNU General Public
@@ -42,16 +42,17 @@
 #include <qradiobutton.h>
 #include <kdebug.h>
 #include <qwidget.h>
+#include <kdevplugininfo.h>
 
-static const KAboutData data("kdevsubversion", I18N_NOOP("Subversion"), "1.0");
+static const KDevPluginInfo data("kdevsubversion");
 
 typedef KDevGenericFactory<subversionPart> subversionFactory;
-K_EXPORT_COMPONENT_FACTORY( libkdevsubversion, subversionFactory( &data ) )
+K_EXPORT_COMPONENT_FACTORY( libkdevsubversion, subversionFactory( data ) )
 
 //bool g_projectWasJustCreated = false;
 
 subversionPart::subversionPart(QObject *parent, const char *name, const QStringList& )
-	: KDevVersionControl("KDevsubversionPart", "kdevsubversionpart", parent, name ? name : "Subversion" ) {
+	: KDevVersionControl(&data, parent, name ? name : "Subversion" ) {
 	setInstance(subversionFactory::instance());
 	m_projWidget = 0;
 
@@ -90,7 +91,6 @@ subversionPart::subversionPart(QObject *parent, const char *name, const QStringL
 	m_impl->processWidget()->setCaption(i18n( "Subversion Output" ));
 	mainWindow()->embedOutputView( (QWidget*)m_impl->processWidget(), i18n( "Subversion" ), i18n( "Subversion messages" ) );
     QWhatsThis::add((QWidget*)m_impl->processWidget(), i18n("<b>Subversion</b><p>Subversion operations window."));
-	setVersionControl( this );
 }
 
 subversionPart::~subversionPart() {
@@ -102,8 +102,7 @@ subversionPart::~subversionPart() {
 }
 
 void subversionPart::setupActions() {
-	actionCommit = new KAction( i18n("&Commit to Repository"), 0, this,
-			SLOT(slotActionCommit()), actionCollection(), "subversion_commit" );
+	actionCommit = new KAction( i18n("&Commit to Repository"), 0, this, SLOT(slotActionCommit()), actionCollection(), "subversion_commit" );
 	actionCommit->setToolTip( i18n("Commit file(s)") );
 	actionCommit->setWhatsThis( i18n("<b>Commit file(s)</b><p>Commits file to repository if modified.") );
 
@@ -112,21 +111,22 @@ void subversionPart::setupActions() {
 				actionDiff->setToolTip( i18n("Build difference") );
 				actionDiff->setWhatsThis( i18n("<b>Build difference</b><p>Builds difference between releases.") );
 				*/
-	actionAdd = new KAction( i18n("&Add to Repository"), 0, this, SLOT(slotActionAdd()),
-			actionCollection(), "subversion_add" );
+	actionAdd = new KAction( i18n("&Add to Repository"), 0, this, SLOT(slotActionAdd()), actionCollection(), "subversion_add" );
 	actionAdd->setToolTip( i18n("Add file to repository") );
 	actionAdd->setWhatsThis( i18n("<b>Add file to repository</b><p>Adds file to repository.") );
 
-	actionRemove = new KAction( i18n("&Remove From Repository"), 0, this,
-			SLOT(slotActionRemove()), actionCollection(), "subversion_remove" );
+	actionRemove = new KAction( i18n("&Remove From Repository"), 0, this, SLOT(slotActionDel()), actionCollection(), "subversion_remove" );
 	actionRemove->setToolTip( i18n("Remove from repository") );
 	actionRemove->setWhatsThis( i18n("<b>Remove from repository</b><p>Removes file(s) from repository.") );
 
-	actionUpdate = new KAction( i18n("&Update"), 0, this,
-			SLOT(slotActionUpdate()), actionCollection(), "subversion_update" );
+	actionUpdate = new KAction( i18n("&Update"), 0, this, SLOT(slotActionUpdate()), actionCollection(), "subversion_update" );
 	actionUpdate->setToolTip( i18n("Update") );
 	actionUpdate->setWhatsThis( i18n("<b>Update</b><p>Updates file(s) from repository.") );
 
+	actionRevert = new KAction( i18n("&Revert"), 0, this, SLOT(slotActionRevert()), actionCollection(), "subversion_revert" );
+	actionRevert->setToolTip( i18n("Revert") );
+	actionRevert->setWhatsThis( i18n("<b>Revert</b><p>Undo local changes.") );
+
 	actionAddToIgnoreList = new KAction( i18n("&Ignore in Subversion Operations"), 0,
 			this, SLOT(slotActionAddToIgnoreList()), actionCollection(), "subversion_ignore" );
 	actionAddToIgnoreList->setToolTip( i18n("Ignore in Subversion operations") );
@@ -136,6 +136,11 @@ void subversionPart::setupActions() {
 			this, SLOT(slotActionRemoveFromIgnoreList()), actionCollection(), "subversion_donot_ignore" );
 	actionRemoveFromIgnoreList->setToolTip( i18n("Do not ignore in Subversion operations") );
 	actionRemoveFromIgnoreList->setWhatsThis( i18n("<b>Do not ignore in Subversion operations</b><p>Do not ignore file(s).") );
+
+	actionResolve = new KAction( i18n("Re&solve Conflicting State"), 0,
+			this, SLOT(slotActionResolve()), actionCollection(), "subversion_resolve" );
+	actionResolve->setToolTip( i18n("Resolve the conflicting state of a file after a merge") );
+	actionResolve->setWhatsThis( i18n("<b>Resolve the conflicting state</b><p>Remove the conflict state that can be set on a file after a merge failed.") );
 }
 
 QWidget* subversionPart::newProjectWidget( QWidget* parent ) {
@@ -145,7 +150,7 @@ QWidget* subversionPart::newProjectWidget( QWidget* parent ) {
 }
 
 void subversionPart::projectConfigWidget( KDialogBase *dlg ) {
-	QVBox *vbox = dlg->addVBoxPage( i18n("Subversion") );
+	QVBox *vbox = dlg->addVBoxPage( i18n("Subversion"), i18n("Subversion"), BarIcon( info()->icon(), KIcon::SizeMedium) );
 	subversionOptionsWidgetImpl *w = new subversionOptionsWidgetImpl( (QWidget *)vbox, this, "svn config widget" );
 	connect( dlg, SIGNAL(okClicked()), w, SLOT(accept()) );
 }
@@ -163,16 +168,7 @@ bool subversionPart::fetchFromRepository() {
 }
 
 KDevVCSFileInfoProvider * subversionPart::fileInfoProvider() const {
-	return 0;
-}
-
-bool subversionPart::isValidDirectory( const QString& dirPath) {
-    QString svn = "/.svn/";
-    QDir svndir( dirPath + svn );
-    QString entriesFileName = dirPath + svn + "entries";
-
-    return svndir.exists() &&
-        QFile::exists( entriesFileName );
+	return m_impl->fileInfoProvider();
 }
 
 void subversionPart::contextMenu( QPopupMenu *popup, const Context *context ) {
@@ -180,21 +176,21 @@ void subversionPart::contextMenu( QPopupMenu *popup, const Context *context ) {
 // want to confuse the user with a SVN popup menu.
 if(!project() || !isValidDirectory(project()->projectDirectory()))
   return;
-  
-	kdDebug() << "contextMenu()" << endl;
+
+	kdDebug(9036) << "contextMenu()" << endl;
 	if (context->hasType( Context::FileContext ) ||
 			context->hasType( Context::EditorContext ))
 	{
 
 		if (context->hasType( Context::FileContext ))
 		{
-			kdDebug() << "Requested for a FileContext" << endl;
+			kdDebug(9036) << "Requested for a FileContext" << endl;
 			const FileContext *fcontext = static_cast<const FileContext*>( context );
 			m_urls = fcontext->urls();
 		}
 		else
 		{
-			kdDebug() << "Requested for an EditorContext" << endl;
+			kdDebug(9036) << "Requested for an EditorContext" << endl;
 			const EditorContext *editorContext = static_cast<const EditorContext*>( context );
 			m_urls << editorContext->url();
 		}
@@ -217,12 +213,16 @@ if(!project() || !isValidDirectory(project()->projectDirectory()))
         subMenu->setWhatsThis(id, i18n("<b>Commit file(s)</b><p>Commits file to repository if modified."));
 		id = subMenu->insertItem( actionAdd->text(), this, SLOT(slotAdd()) );
         subMenu->setWhatsThis(id, i18n("<b>Add file to repository</b><p>Adds file to repository."));
-		id = subMenu->insertItem( actionRemove->text(), this, SLOT(slotRemove()) );
+		id = subMenu->insertItem( actionRemove->text(), this, SLOT(slotDel()) );
         subMenu->setWhatsThis(id, i18n("<b>Remove from repository</b><p>Removes file(s) from repository."));
 
 		subMenu->insertSeparator();
 		id = subMenu->insertItem( actionUpdate->text(), this, SLOT(slotUpdate()) );
         subMenu->setWhatsThis(id, i18n("<b>Update</b><p>Updates file(s) from repository."));
+		id = subMenu->insertItem( actionRevert->text(), this, SLOT(slotRevert()) );
+		subMenu->setWhatsThis(id, i18n("<b>Revert</b><p>Undo local changes.") );
+		id = subMenu->insertItem( actionResolve->text(), this, SLOT(slotResolve()) );
+		subMenu->setWhatsThis(id, i18n("<b>Resolve</b><p>Resolve conflicting state.") );
 
 		subMenu->insertSeparator();
 		id = subMenu->insertItem( actionAddToIgnoreList->text(), this, SLOT(slotAddToIgnoreList()) );
@@ -247,7 +247,7 @@ bool subversionPart::urlFocusedDocument( KURL &url ) {
 }
 
 void subversionPart::slotActionUpdate() {
-	kdDebug() << "subversion: slotActionUpdate()" << endl;
+	kdDebug(9036) << "subversion: slotActionUpdate()" << endl;
 	KURL doc;
 	if (urlFocusedDocument( doc )) {
 		m_impl->update( doc );
@@ -258,20 +258,68 @@ void subversionPart::slotUpdate() {
 	m_impl->update (m_urls);
 }
 
+void subversionPart::slotActionResolve() {
+	kdDebug(9036) << "subversion: slotActionResolve()" << endl;
+	KURL doc;
+	if (urlFocusedDocument( doc )) {
+		m_impl->resolve( doc );
+	}
+}
+
+void subversionPart::slotResolve() {
+	m_impl->resolve (m_urls);
+}
+
 void subversionPart::slotActionCommit() {
-	kdDebug() << "subversion: slotActionCommit()" << endl;
+	kdDebug(9036) << "subversion: slotActionCommit()" << endl;
 	KURL doc;
 	if (urlFocusedDocument( doc )) {
 		m_impl->commit( doc );
 	}
 }
 
+void subversionPart::slotActionAdd() {
+	kdDebug(9036) << "subversion: slotActionAdd()" << endl;
+	KURL doc;
+	if (urlFocusedDocument( doc )) {
+		m_impl->add( doc );
+	}
+}
+
+void subversionPart::slotActionDel() {
+	kdDebug(9036) << "subversion: slotActionDel()" << endl;
+	KURL doc;
+	if (urlFocusedDocument( doc )) {
+		m_impl->del( doc );
+	}
+}
+
+void subversionPart::slotActionRevert() {
+	kdDebug(9036) << "subversion: slotActionRevert()" << endl;
+	KURL doc;
+	if (urlFocusedDocument( doc )) {
+		m_impl->revert( doc );
+	}
+}
+
 void subversionPart::slotCommit() {
 	m_impl->commit (m_urls);
 }
 
+void subversionPart::slotAdd() {
+	m_impl->add (m_urls);
+}
+
+void subversionPart::slotDel() {
+	m_impl->del (m_urls);
+}
+
+void subversionPart::slotRevert() {
+	m_impl->revert (m_urls);
+}
+
 void subversionPart::slotProjectOpened() {
-	kdDebug() << "subversion :projectOpened" << endl;
+	kdDebug(9036) << "subversion :projectOpened" << endl;
 /*	if ( g_projectWasJustCreated ) {
 		//saveOptions();
 		g_projectWasJustCreated = false;
@@ -283,7 +331,7 @@ void subversionPart::slotProjectOpened() {
 }
 
 void subversionPart::slotProjectClosed() {
-	kdDebug() << "subversion :projectClosed" << endl;
+	kdDebug(9036) << "subversion :projectClosed" << endl;
 	//saveOptions();
 	/// \FIXME slots
 	//disconnect( project(), SIGNAL(addedFilesToProject(const QStringList&)), this, SLOT(slotAddFilesToProject(const QStringList &)) );
@@ -291,7 +339,7 @@ void subversionPart::slotProjectClosed() {
 }
 
 void subversionPart::savePartialProjectSession(QDomElement* dom) {
-	kdDebug() << "subversion : savePartialProjectSession" << endl;
+	kdDebug(9036) << "subversion : savePartialProjectSession" << endl;
 	QDomDocument doc = dom->ownerDocument();
 	QDomElement svn = doc.createElement( "subversion" );
 	svn.setAttribute("recursecheckout", m_checkout_recurse);
@@ -315,7 +363,7 @@ void subversionPart::savePartialProjectSession(QDomElement* dom) {
 }
 
 void subversionPart::restorePartialProjectSession(const QDomElement* dom) {
-	kdDebug() << "subversion : restorePartialProjectSession" << endl;
+	kdDebug(9036) << "subversion : restorePartialProjectSession" << endl;
 	QDomElement svn = dom->namedItem("subversion").toElement();
 
 	m_checkout_recurse = svn.attribute( "recursecheckout", "1" ).toInt();
@@ -337,4 +385,15 @@ void subversionPart::restorePartialProjectSession(const QDomElement* dom) {
 	base = svn.attribute( "base", "" );
 }
 
+bool subversionPart::isValidDirectory( const QString &dirPath) const {
+    QString svn = "/.svn/";
+    QDir svndir( dirPath + svn );
+    QString entriesFileName = dirPath + svn + "entries";
+
+	kdDebug(9036) << "dirpath " << dirPath+"/.svn/" << svndir.exists() << endl;
+	kdDebug(9036) << "entries " << entriesFileName << QFile::exists( entriesFileName ) << endl;
+    return svndir.exists() &&
+        QFile::exists( entriesFileName );
+}
+
 #include "subversion_part.moc"
